/***********************************************************************

  ColorBox2: make.cpp
    (c) 2004 by Martin Decky

  Just-In-Time compiler, runs 'make' to build the user's binary.
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			10th August 2004	initial

***********************************************************************/


#include <iostream>
#include <unistd.h>
#include "make.h"
#include "../config.h"


using namespace std;


/** Remove tailing slash
 *
 * Remove tailing slash (if present) from path string.
 *
 * @param path Path string.
 *
 * @return Path w/o tailing slash.
 *
 */
String unslash(const String & path) {
	if ((path.Len() > 0) && (path[path.Len() - 1] == '/'))
		return path.Substr(0, path.Len() - 1);
	
	return path;
}


/** Run 'make' to compile the binary
 *
 * Run 'make' to compile the source files
 * according to local Makefile to binary.
 *
 * The 'make' should be in path. The first
 * parameter is BIN_PATH=$bin_path (describing
 * the binary destination path), the second
 * parameter is CB2_FLAGS=$flags and the third
 * parameter is the target (w/o path).
 *
 * If the execution was succesful, this function
 * doesn't return.
 *
 * @param target   Binary target to make.
 * @param bin_path Path of the binary output directory.
 *
 * @return Non-zero for execution error.
 *
 */
int make(const String & target, const String & bin_path) {
	/* Execute make */
	if (execlp("make", "make", (const char * ) (String("BIN_PATH=") + unslash(bin_path)), "CB2_FLAGS=" CB2_FLAGS, (const char * ) target, NULL) == -1)
		cerr << "Unable to execute make" << endl;
	
	/* Normally we shouldn't reach
	   this code */
	cerr << "Execution error" << endl;
	return 3;
}
