/***********************************************************************

  ColorBox2: utils.cpp
    (c) 2004 by Martin Decky

  Common datatypes, classes, macros.
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			10th August 2004	conversion from C to C++

***********************************************************************/


/** @mainpage
 *
 * ColorBox2 is an application framework for developing and deploying
 * fast web applications written in C++. This class reference contains
 * description of all the classes and methods available to the end
 * programmer in the libcb2 library.
 *
 * To understand the parts of the framework in the bigger scale and additional
 * information, please refer to the <a href="../../index.html">human-written documentation</a>.
 *
 */

#include <cb2/utils.h>


using namespace CB2;


/** Constructor
 *
 * Generic exception without description.
 *
 */
Exception::Exception() {
	except = "";
}


/** Constructor with description
 *
 * Generic exception with description.
 *
 * @param what Exception description.
 *
 */
Exception::Exception(const String what) {
	except = what;
}


/** Constructor with description and line number
 *
 * Generic exception with description and line
 * number (in the source code) where the exception
 * occured.
 *
 * @param what Exception description.
 * @param line Line number.
 *
 */
Exception::Exception(const String what, const int line) {
	except = what + String(" (line ") + String(line) + String(")");
}
			

/** Get the exception description
 *
 * @return Exception description.
 *
 */
String Exception::What() const {
	return except;
}


/** Constructor
 *
 * Conversion exception without description.
 *
 */
ExceptionConversion::ExceptionConversion()
	: Exception() {
	/* Empty */
}


/** Constructor with description
 *
 * Conversion exception with description.
 *
 * @param what Exception description.
 *
 */
ExceptionConversion::ExceptionConversion(const String what)
	: Exception(what) {
	/* Empty */
}


/** Constructor with description and line number
 *
 * Conversion exception with description and line
 * number (in the source code) where the exception
 * occured.
 *
 * @param what Exception description.
 * @param line Line number.
 *
 */
ExceptionConversion::ExceptionConversion(const String what, const int line)
	: Exception(what, line) {
	/* Empty */
}


/** Constructor
 *
 * Memory access exception without description.
 *
 */
ExceptionMemory::ExceptionMemory()
	: Exception() {
	/* Empty */
}


/** Constructor with description
 *
 * Memory access exception with description.
 *
 * @param what Exception description.
 *
 */
ExceptionMemory::ExceptionMemory(const String what)
	: Exception(what) {
	/* Empty */
}


/** Constructor with description and line number
 *
 * Memory access exception with description and line
 * number (in the source code) where the exception
 * occured.
 *
 * @param what Exception description.
 * @param line Line number.
 *
 */
ExceptionMemory::ExceptionMemory(const String what, const int line)
	: Exception(what, line) {
	/* Empty */
}


/** Constructor
 *
 * Assertion failed exception without description.
 *
 */
ExceptionAssert::ExceptionAssert()
	: Exception() {
	/* Empty */
}


/** Constructor with description
 *
 * Assertion failed exception with description.
 *
 * @param what Exception description.
 *
 */
ExceptionAssert::ExceptionAssert(const String what)
	: Exception(what) {
	/* Empty */
}


/** Constructor with description and line number
 *
 * Assertion failed exception with description and line
 * number (in the source code) where the exception
 * occured.
 *
 * @param what Exception description.
 * @param line Line number.
 *
 */
ExceptionAssert::ExceptionAssert(const String what, const int line)
	: Exception(what, line) {
	/* Empty */
}


/** Constructor
 *
 * Item not found exception without description.
 *
 */
ExceptionNotFound::ExceptionNotFound()
	: Exception() {
	/* Empty */
}


/** Constructor with description
 *
 * Item not found exception with description.
 *
 * @param what Exception description.
 *
 */
ExceptionNotFound::ExceptionNotFound(const String what)
	: Exception(what) {
	/* Empty */
}


/** Constructor with description and line number
 *
 * Item not found exception with description and line
 * number (in the source code) where the exception
 * occured.
 *
 * @param what Exception description.
 * @param line Line number.
 *
 */
ExceptionNotFound::ExceptionNotFound(const String what, const int line)
	: Exception(what, line) {
	/* Empty */
}


template class PString< char>;            /**< Explicit template instantiation */
template class PStringTokenizer< char>;   /**< Explicit template instantiation */
template class PMap< String, String>;     /**< Explicit template instantiation */
template class PMapList< String, String>; /**< Explicit template instantiation */
