/***********************************************************************

  ColorBox2: Application.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.2			20th June 2003		basic structure
  0.3			20th August 2004	rewrite

***********************************************************************/


#include <iostream>
#include <cb2/Application.h>


using namespace CB2;
using namespace std;


Serializer * global_serializer = NULL;


Serializer::Serializer(int argc, char * argv[], std::istream & input) {
	CB2_ASSERT(global_serializer == NULL, "Another serializer already instantiated");
	load(argc, argv, input);
	global_serializer = this;
}


Serializer::~Serializer() {
	global_serializer = NULL;
}


void Serializer::BeginInstance(const char * serialname, const char * instance) {
}


const Serializer & Serializer::operator<<(const String & data) {
	return *this;
}


const Serializer & Serializer::operator>>(String & data) {
	return *this;
}


void Serializer::EndInstance() {
}
			

String Serializer::Render() {
	return String(""); // FIXME
}


Serializer & Serializer::GetSerializer() {
	CB2_ASSERT(global_serializer != NULL, "No serializer instantiated");
	return *global_serializer;
}


void Serializer::load(int argc, char * argv[], std::istream & input) {
	if (argc > 2) {
		StringTokenizer params(argv[2], "&");
		
		for (StringTokenizer::iterator it = params.Begin(); it != params.End(); ++it) {
			StringTokenizer map(*it, "=");
			
			if ((map.Len() == 2) && (map[0] != ""))
				GetVars.Add(map[0].URLdecode(), map[1].URLdecode());
		}
	}
	
	std::string line;
	while (std::getline(cin, line, '\n')) {
		StringTokenizer params(line, "&");
		
		for (StringTokenizer::iterator it = params.Begin(); it != params.End(); ++it) {
			StringTokenizer map(*it, "=");
			
			if ((map.Len() == 2) && (map[0] != ""))
				PostVars.Add(map[0].URLdecode(), map[1].URLdecode());
		}
	}
}


Application::Application() {
	templ = NULL;
	mime = "text/html";
	member = NULL;
}


Application::~Application() {
	if (templ) {
		String serial_data = Serializer::GetSerializer().Render();
		if (serial_data != "") {
			templ->Subst("SERIAL_DATA_HTML", serial_data.HTML());
			templ->SubstURL("SERIAL_DATA_URL", serial_data);
		}
	
		cout << mime << endl;
		
		if (member)
			delete member;
		else
			templ->Subst(anchor, "");
			
		cout << templ->Render();
		delete templ;
	} else {
		cout << "text/html" << endl;
		cout << "<html>" << endl
			<< "<head>" << endl
			<< " <title>Default ColorBox2 Application</title>" << endl
			<< "</head>" << endl
			<< "<body>" << endl
			<< "<h1>Default ColorBox2 Application</h1>" << endl
			<< "<p>Default CB2::Application class used.</p>" << endl
			<< "</body>" << endl
			<< "</html>" << endl;
	}
}


Template * Application::Register(Widget * child, String & anc) {
	if (child == NULL)
		return NULL;
	
	CB2_ASSERT(member == NULL, "CB2::Application can contain just one container");
	
	member = child;
	anc = anchor;
	return templ;
}


void Application::SetTemplate(Template * temp, const String & anc) {
	templ = temp;
	anchor = anc;
}


void Application::SetMimeType(const String & mtype) {
	mime = mtype;
}


void Application::AppError(const String & error, const String & desc) {
	cout << "text/html" << endl
		<< "<html>" << endl
		<< "<head>" << endl
		<< " <title>" << error.HTML() << "</title>" << endl
		<< "</head>" << endl
		<< "<body>" << endl
		<< "<h1>" << error.HTML() << "</h1>" << endl
		<< "<p>";
	
	if (desc == "")
		cout << "See debug output";
	else
		cout << desc;
		
	cout << "</p>" << endl
		<< "</body>" << endl
		<< "</html>" << endl;
}
