/***********************************************************************

  ColorBox2: DS.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.4			10th September 2004		initial

***********************************************************************/


#include <cb2/DS.h>


using namespace CB2;


/** Constructor
 *
 */
DataSource::DataSource() {
}


/** Destructor
 *
 */
DataSource::~DataSource() {
}


/** Default constructor
 *
 */
ExceptionMySQL::ExceptionMySQL()
	: Exception() {
	err = unknown_error;
}


/** Costructor with error type and description
 *
 */
ExceptionMySQL::ExceptionMySQL(const error_type error, const String what)
	: Exception(what) {
	err = error;
}


/** Costructor with error type, description and line
 *
 */
ExceptionMySQL::ExceptionMySQL(const error_type error, const String what, const int line)
	: Exception(what, line) {
	err = error;
}
			

/** Get error
 *
 */
ExceptionMySQL::error_type ExceptionMySQL::GetError() const {
	return err;
}


DSMySQL::DSMySQL(const String & host, const String & user, const String & passwd, const String & db, const unsigned int port, const String & socket)
	: DataSource() {
	
	/* Initialize handle */
	handle = mysql_init(NULL);
	CB2_MYSQL(handle != NULL, internal_error, "Unable to create MySQL handle");
	
	/* Connect to server */
	handle = mysql_real_connect(handle, host, user, passwd, db, port, socket, 0);
	CB2_MYSQL(handle != NULL, connection_error, "Unable to connect to MySQL database");
}


DSMySQL::~DSMySQL() {
	mysql_close(handle);
}


Query DSMySQL::Select(const String & table, const Domain & domain, const Query::size_type limit, const Query::size_type displacement) {
	String sql = String("SELECT * FROM '") + table.SQL() + String("'");
	
	/* Costruct WHERE clause of the SELECT
	   query */
	bool f_lit = false;
	for (Domain::constrain_cnf::iterator literal = domain.constrains.Begin(); literal != domain.constrains.End(); ++literal) {
		/* Go through all literals */
		if (!f_lit)
			sql += " WHERE";
		
		bool f_at = false;
		for (Domain::constrain_literal::iterator atom = literal->Begin(); atom != literal->End(); ++atom) {
			/* Go throught all atoms */
			if (!f_at) {
				if (f_lit)
					sql += " AND";
				sql += " (";
			} else
				sql += " OR";
			
			switch (atom->type) {
				case Domain::str_value: /* String value constrains */
					switch (atom->str_value.con) {
						case Domain::str_row_eq: /* Equals */
							sql += String(" '") + atom->str_value.key.SQL() + String("' = \"") + atom->str_value.value.SQL() + String("\"");
							break;
						case Domain::str_row_neq: /* Not equals */
							sql += String(" '") + atom->str_value.key.SQL() + String("' != \"") + atom->str_value.value.SQL() + String("\"");
							break;
						default:
							CB2_ASSERT(false, "Unknown domain string value constrain");
					}
					break;
				case Domain::int_value: /* Integer value constrains */
					switch (atom->int_value.con) {
						case Domain::int_row_eq: /* Equals */
							sql += String(" '") + atom->int_value.key.SQL() + String("' = ") + String(atom->int_value.value).SQL();
							break;
						case Domain::int_row_neq: /* Not equals */
							sql += String(" '") + atom->int_value.key.SQL() + String("' != ") + String(atom->int_value.value).SQL();
							break;
						case Domain::int_row_gt: /* Greater than */
							sql += String(" '") + atom->int_value.key.SQL() + String("' > ") + String(atom->int_value.value).SQL();
							break;
						case Domain::int_row_lt: /* Less than */
							sql += String(" '") + atom->int_value.key.SQL() + String("' < ") + String(atom->int_value.value).SQL();
							break;
						case Domain::int_row_ge: /* Greater or equal */
							sql += String(" '") + atom->int_value.key.SQL() + String("' >= ") + String(atom->int_value.value).SQL();
							break;
						case Domain::int_row_le: /* Less or equal */
							sql += String(" '") + atom->int_value.key.SQL() + String("' <= ") + String(atom->int_value.value).SQL();
							break;
						default:
							CB2_ASSERT(false, "Unknown domain integer value constrain");
					}
					break;
				default:
					CB2_ASSERT(false, "Unknown domain atom type");
			}
			
			f_at = true;
		}
		if (f_at)
			sql += ")";
		
		f_lit = true;
	}
	
	/* Run SQL query */
	int ret = mysql_real_query(handle, sql, sql.Len());
	CB2_MYSQL(ret == 0, query_error, "Query failed");
	
	/* Fetch result set */
	MYSQL_RES * result = mysql_store_result(handle);
	CB2_MYSQL(result != NULL, query_error, "Empty result set");
	
	unsigned int nfields = mysql_num_fields(result);   /* Number of result fields */
	MYSQL_FIELD * fields = mysql_fetch_fields(result); /* Result fields names */
	CB2_ASSERT(fields != NULL, "Field descriptions are NULL");
	
	Query query;
	MYSQL_ROW row;
	
	/* Parse result set */
	while ((row = mysql_fetch_row(result)) != NULL) {
		unsigned long * lengths = mysql_fetch_lengths(result);
		CB2_ASSERT(lengths != NULL, "Field lengths are NULL");
		Row qrow;
		
		/* Parse fields of a single row */
		for (unsigned int field = 0; field < nfields; field++) {
			if (row[field] == NULL)
				qrow.Add(fields[field].name);
			else
				qrow.Add(fields[field].name, String(row[field], lengths[field]));
		}
		
		query.PushBack(qrow);
	}
	
	mysql_free_result(result);
	
	return query;
}
