/***********************************************************************

  ColorBox2: Forms.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.4			10th September 2004		initial

***********************************************************************/


#include <cb2/Forms.h>


using namespace CB2;


Form::Form(Container & parent, const String & css_class, const String & css_id)
	: Widget(parent) {
	init(css_class, css_id);
}


Form::Form(Application & parent, const String & css_class, const String & css_id)
	: Widget(parent) {
	init(css_class, css_id);
}


Form::~Form() {
	(*mtempl) += "</form>";
	
	for (element_list::iterator it = elements.Begin(); it != elements.End(); ++it)
		delete &(*it);
		
	Data(mtempl->Render());
	delete mtempl;
}


Template * Form::Add(FormElement * child, String & anchor) {
	anchor = String("INPUT") + String(elements.Len());
	(*mtempl) += String("&&") + anchor + String(";");
	elements << child;
	
	return mtempl;
}


void Form::init(const String & css_class, const String & css_id) {
	String tmp = "<form action=\"\" method=\"post\"";
	
	if ((css_class != "") || (css_id != "")) {
		if (css_id != "")
			tmp += String(" id=\"") + css_id.HTML() + String("\"");
		
		if (css_class != "")
			tmp += String(" class=\"") + css_class.HTML() + String("\"");
		
		tmp += ">";
	}

	mtempl = new Template(tmp);
}


FormElement::FormElement(Form & parent) {
	ptempl = parent.Add(this, panchor);
	filled = false;
}


void FormElement::Data(const String & data) {
	filled = true;
	if (ptempl)
		ptempl->Subst(panchor, data);
}


FormElement::~FormElement() {
	CB2_ASSERT(filled, "FormElement didn't supply its data to parent");
}


FormElement::FormElement(const FormElement & prev) {
	CB2_ASSERT(false, "FormElement cannot use copy constructor");
}


void FormElement::operator=(const FormElement & prev) {
	CB2_ASSERT(false, "FormElement cannot use assignment operator");
}


Button::Button(Form & parent, const button_type & type, const String & caption, const String & name, const String & css_class, const String & css_id)
	: FormElement(parent) {
	_type = type;
	_caption = caption;
	_name = name;
	_css_class = css_class;
	_css_id = css_id;
}


Button::~Button() {
	String tmp;
	
	switch (_type) {
		case button_submit:
			tmp = String("<input type=\"submit\" value=\"") + _caption.HTML() + String("\"");
			if (_name != "")
				tmp += String(" name=\"") + _name.HTML() + String("\"");
			tmp += " />";
			break;
		case button_reset:
			tmp = String("<input type=\"reset\" value=\"") + _caption.HTML() + String("\"");
			if (_name != "")
				tmp += String(" name=\"") + _name.HTML() + String("\"");
			tmp += " />";
			break;
		default:
			CB2_ASSERT(false, "Unknown button type");
	}

	Data(tmp);
}


ListBox::ListBox(Form & parent, const String & name, const MapList & options, const String & selected, const String & css_class, const String & css_id)
	: FormElement(parent) {
	_name = name;
	_options = options;
	_selected = selected;
	_css_class = css_class;
	_css_id = css_id;
}


ListBox::~ListBox() {
	String tmp = String("<select name=\"") + _name.HTML() + String("\"");
	
	if (_css_id != "")
		tmp += String(" id=\"") + _css_id.HTML() + String("\"");
	
	if (_css_class != "")
		tmp += String(" class=\"") + _css_class.HTML() + String("\"");
		
	tmp += ">";
	
	for (MapList::iterator it = _options.Begin(); it != _options.End(); ++it) {
		tmp += String("<option value=\"") + it->GetKey().HTML() + String("\"");
		if (it->GetKey() == _selected)
			tmp += " selected=\"selected\"";
		tmp += ">";
		if (!it->IsNULL())
			tmp += it->GetValue().HTML();
		tmp += "</option>";
	}
	
	tmp += "</select>";
	
	Data(tmp);
}


InputBox::InputBox(Form & parent, const String & name, const String & value, const String & css_class, const String & css_id)
	: FormElement(parent) {
	_name = name;
	_value = value;
	_css_class = css_class;
	_css_id = css_id;
}


InputBox::~InputBox() {
	String tmp;
	
	if (_css_id != "")
		tmp += String(" id=\"") + _css_id.HTML() + String("\"");
	
	if (_css_class != "")
		tmp += String(" class=\"") + _css_class.HTML() + String("\"");
	
	Data(String("<input type=\"text\" name=\"") + _name.HTML() + String("\" value=\"") + _value.HTML() + String("\"") + tmp + String(" />"));
}
