/***********************************************************************

  ColorBox2: Forms.h
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.4			10th September 2004		initial

***********************************************************************/


#ifndef __CB2__FORMS_H__
#define __CB2__FORMS_H__


#include <cb2/utils.h>
#include <cb2/Widgets.h>
#include <cb2/Containers.h>


namespace CB2 {
	
	class FormElement;
	
	class Form : public Widget {
		public:
			Form(Container & parent, const String & css_class = "", const String & css_id = "");
			Form(Application & parent, const String & css_class = "", const String & css_id = "");
			
			virtual ~Form();
			
			virtual Template * Add(FormElement * child, String & anchor);
			
		private:
			typedef List< FormElement> element_list;
			
			Template * mtempl;
			element_list elements;
			
			void init(const String & css_class, const String & css_id);
	};
	
	
	class FormElement : public Class {
		public:
			FormElement(Form & parent);
			virtual ~FormElement();
			
		protected:
			void Data(const String & data);
			
		private:
			FormElement(const FormElement & prev);
			void operator=(const FormElement & prev);
			
			Template * ptempl;
			String panchor;
			bool filled;
	};
	
	
	class Button : public FormElement {
		public:
			
			typedef enum {
				button_submit,
				button_reset
			} button_type;
			
			Button(Form & parent, const button_type & type, const String & caption, const String & name = "", const String & css_class = "", const String & css_id = "");
			virtual ~Button();
			
		private:
			button_type _type;
			String _caption;
			String _name;
			String _css_class;
			String _css_id;
	};
	
	
	class ListBox : public FormElement {
		public:
			
			ListBox(Form & parent, const String & name, const MapList & options, const String & selected = "", const String & css_class = "", const String & css_id = "");
			virtual ~ListBox();
			
		private:
			String _name;
			MapList _options;
			String _selected;
			String _css_class;
			String _css_id;
	};
	
	
	class InputBox : public FormElement {
		public:
			InputBox(Form & parent, const String & name, const String & value, const String & css_class = "", const String & css_id = "");
			virtual ~InputBox();
			
		private:
			String _name;
			String _value;
			String _css_class;
			String _css_id;
	};
	
}


#endif
