/***********************************************************************

  ColorBox2: Paragraph.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			20th August 2004	initial

***********************************************************************/


#include <cb2/Paragraph.h>


using namespace CB2;


/** Constructor
 *
 * Create an anchor.
 *
 * @param url       Anchor URL.
 * @param text      Anchor text.
 * @param title     Anchor title.
 * @param css_class Anchor CSS class.
 * @param css_id    Anchor CSS id.
 *
 */
Anchor::Anchor(const String & url, const String & text, const String & title, const String & css_class, const String & css_id) {
	_url = url;
	_text = text;
	_title = title;
	_css_class = css_class;
	_css_id = css_id;
}


const String & Anchor::GetUrl() const {
	return _url;
}


const String & Anchor::GetText() const {
	return _text;
}


const String & Anchor::GetTitle() const {
	return _title;
}


const String & Anchor::GetCSSclass() const {
	return _css_class;
}


const String & Anchor::GetCSSid() const {
	return _css_id;
}


Text::Text() {
	text_component initial;
	initial.html = false;
	components.PushBack(initial);
}


Text::Text(const Text & prev) {
	// TODO
}

			
const Text & Text::operator=(const Text & prev) {
	// TODO
	return *this;
}


Text & Text::operator<<(const String & text) {
	text_component * tail = components.Tail();
	CB2_ASSERT(tail != NULL, "Inconsistent text components");
	
	tail->text += text;
	return *this;
}


Text & Text::operator<<(const Anchor & anchor) {
	text_component * tail = components.Tail();
	CB2_ASSERT(tail != NULL, "Inconsistent text components");
	
	text_component comp;
	comp.html = tail->html;
	
	text_component ncomp;
	ncomp.html = true;
	
	ncomp.text += String("<a href=\"") + anchor.GetUrl().HTML() + String("\"");
	
	if (anchor.GetTitle() != "")
		ncomp.text += String(" title=\"") + anchor.GetTitle().HTML() + String("\"");
	
	if (anchor.GetCSSclass() != "")
		ncomp.text += String(" class=\"") + anchor.GetCSSclass().HTML() + String("\"");
	
	if (anchor.GetCSSid() != "")
		ncomp.text += String(" class=\"") + anchor.GetCSSid().HTML() + String("\"");

	ncomp.text += String(">") + anchor.GetText().HTML() + String("</a>");
	
	components.PushBack(ncomp);
	components.PushBack(comp);
	
	return *this;
}


Text & Text::operator<<(const text_cmd & cmd) {
	text_component * tail = components.Tail();
	CB2_ASSERT(tail != NULL, "Inconsistent text components");
	
	text_component comp;
	comp.html = tail->html;
	
	text_component ncomp;
	ncomp.html = true;
	
	switch (cmd) {
		case strong:
			ncomp.text = "<strong>";
			break;
		case no_strong:
			ncomp.text = "</strong>";
			break;
		case em:
			ncomp.text = "<em>";
			break;
		case no_em:
			ncomp.text = "</em>";
			break;
		case tt:
			ncomp.text = "<tt>";
			break;
		case no_tt:
			ncomp.text = "</tt>";
			break;
		default:
			CB2_ASSERT(false, "Unknown text command");
	}
	
	components.PushBack(ncomp);
	components.PushBack(comp);
	
	return *this;
}


String Text::HTML() const {
	String ret;
	
	for (text_components::iterator it = components.Begin(); it != components.End(); ++it) {
		if (it->html)
			ret += it->text;
		else
			ret += it->text.HTML("<br />");
	}
	
	return ret;
}


Paragraph::Paragraph(Container & parent, const String & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_css_class = css_class;
	_css_id = css_id;
}


Paragraph::Paragraph(Application & parent, const String & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_css_class = css_class;
	_css_id = css_id;
}


Paragraph::Paragraph(Container & parent, const Text & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_css_class = css_class;
	_css_id = css_id;
}


Paragraph::Paragraph(Application & parent, const Text & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_css_class = css_class;
	_css_id = css_id;
}


Paragraph::~Paragraph() {
	String tmp;
	
	if (_css_id != "")
		tmp += String(" id=\"") + _css_id.HTML() + String("\"");
	
	if (_css_class != "")
		tmp += String(" class=\"") + _css_class.HTML() + String("\"");
	
	Data(String("<p") + tmp + String(">") + contents + String("</p>"));
}


Heading::Heading(Container & parent, const unsigned int level, const String & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_level = level;
	_css_class = css_class;
	_css_id = css_id;
}


Heading::Heading(Application & parent, const unsigned int level, const String & text, const String & css_class, const String & css_id)
	: Widget(parent) {
	contents = text.HTML();
	_level = level;
	_css_class = css_class;
	_css_id = css_id;
}


Heading::~Heading() {
	String tmp;
	String tmp_lev = String(_level).HTML();
	
	if (_css_id != "")
		tmp += String(" id=\"") + _css_id.HTML() + String("\"");
	
	if (_css_class != "")
		tmp += String(" class=\"") + _css_class.HTML() + String("\"");
	
	Data(String("<h") + tmp_lev + tmp + String(">") + contents + String("</h") + tmp_lev + String(">"));
}
