/***********************************************************************

  ColorBox2: Paragraph.h
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			20th August 2004	initial

***********************************************************************/


#ifndef __CB2__PARAGRAPH_H__
#define __CB2__PARAGRAPH_H__


#include <cb2/utils.h>
#include <cb2/Widgets.h>


#define STRONG CB2::Text::strong
#define NO_STRONG CB2::Text::no_strong
#define EM CB2::Text::em
#define NO_EM CB2::Text::no_em
#define TT CB2::Text::tt
#define NO_TT CB2::Text::no_tt


namespace CB2 {
	
	
	/** Anchor
	 *
	 * Class representing HTML anchor.
	 *
	 */
	class Anchor : public Class {
		public:
			Anchor(const String & url, const String & text, const String & title = "", const String & css_class = "", const String & css_id = ""); /**< Constructor */
			
			const String & GetUrl() const;      /**< Get anchor URL */
			const String & GetText() const;     /**< Get anchor text */
			const String & GetTitle() const;    /**< Get anchor title */
			const String & GetCSSclass() const; /**< Get anchor CSS class */
			const String & GetCSSid() const;    /**< Get anchor CSS id */
		
		private:
			
			String _url;       /* Anchor URL */
			String _text;      /* Anchor text */
			String _title;     /* Anchor title */
			String _css_class; /* Anchor CSS class */
			String _css_id;    /* Anchor CSS id */
		
	};
	
	
	/** Rich text storage
	 *
	 * Class for storing rich text.
	 *
	 */
	class Text : public Class {
		public:
			
			/** Text commands
			 *
			 * Commands representing logical text styles.
			 *
			 */
			typedef enum {
				strong,    /**< Strong (bold) */
				no_strong, /**< Strong off */
				em,        /**< Emphasis (italics) */
				no_em,     /**< Emphasis off */
				tt,        /**< Preformatted */
				no_tt      /**< Preformatted off */
			} text_cmd;
		
			Text();                  /**< Constructor */
			Text(const Text & prev); /**< Copy constructor */
			
			const Text & operator=(const Text & prev); /**< Assignment operator */
			Text & operator<<(const String & text);    /**< Concatenation */
			Text & operator<<(const Anchor & anchor);  /**< Concatenation */
			Text & operator<<(const text_cmd & cmd);   /**< Concatenation */
			
			String HTML() const; /**< Render HTML output */
	
		private:
			
			/** One component of rich text
			 *
			 */
			typedef struct {
				bool html;   /**< Direct (HTML) value */
				String text; /**< Text value */
			} text_component;
			
			typedef List< text_component> text_components; /**< Components of rich text type */
			
			text_components components; /**< Components of rich text */
	};

	
	/** Widget for paragraph text
	 *
	 * Widget represented by a single paragraph
	 * of text or rich text.
	 *
	 */
	class Paragraph : public Widget {
		public:
			Paragraph(Container & parent, const String & text, const String & css_class = "", const String & css_id = "");   /**< Constructor */
			Paragraph(Application & parent, const String & text, const String & css_class = "", const String & css_id = ""); /**< Constructor */
			
			Paragraph(Container & parent, const Text & text, const String & css_class = "", const String & css_id = "");     /**< Constructor */
			Paragraph(Application & parent, const Text & text, const String & css_class = "", const String & css_id = "");   /**< Constructor */
			
			virtual ~Paragraph(); /**< Destructor */
			
		private:
			String contents;   /**< Paragraph contents */
			String _css_class; /**< CSS class */
			String _css_id;    /**< CSS id */
	};
	
	
	/** Widget for heading
	 *
	 * Widget represented by a heading.
	 *
	 */
	class Heading : public Widget {
		public:
			Heading(Container & parent, const unsigned int level, const String & text, const String & css_class = "", const String & css_id = "");   /**< Constructor */
			Heading(Application & parent, const unsigned int level, const String & text, const String & css_class = "", const String & css_id = ""); /**< Constructor */
			
			virtual ~Heading(); /**< Destructor */
			
		private:
			String contents;     /**< Heading contents */
			unsigned int _level; /**< Heading level */
			String _css_class;   /**< CSS class */
			String _css_id;      /**< CSS id */
	};
	
}


#endif
