/***********************************************************************

  ColorBox2: Template.cpp
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.2			20th June 2003		basic structure
  0.3			20th August 2004	rewrite

***********************************************************************/


#include "Template.h"


using namespace CB2;


/** Constructor
 *
 * Create an empty template.
 *
 */
Template::Template() {
}


/** Constructor
 *
 * Create a template initialized from string.
 *
 * @param value Template value.
 *
 */
Template::Template(const String & value) {
	macro = value;
}


/** Append a string to template
 *
 * @param right String to append.
 *
 * @return Reference to this.
 *
 */
const Template & Template::operator+=(const String & right) {
	macro += right;
	return *this;
}


/** Substitute an anchor in template
 *
 * Substitute an anchor in the template with another
 * value.
 *
 * @param anchor Anchor name to substitute (the actual substituted
 *               string is "&&anchor;").
 * @param value  Value to substitute.
 *
 * @return Number of substitutions done.
 *
 */
String::size_type Template::Subst(const String & anchor, const String & value) {
	return macro.Subst(String("&&") + anchor + String(";"), value);
}


/** Substitute an URL anchor in template
 *
 * TODO
 *
 * @param anchor Anchor name to substitute.
 * @param value  Value to substitute.
 *
 * @return Number of substitutions done.
 *
 */
String::size_type Template::SubstURL(const String & anchor, const String & value) {
	return 0; // FIXME
}


/** Render template
 *
 * Generate the final output of the template. Check for
 * any unsubstituted anchors and remove escaped ampersands.
 *
 * @return Rendered template.
 *
 */
String Template::Render() {
	/* Work on a copy of template */
	String temp = macro;

	/* Check for any &&anchor left */
	String::size_type pos;
	while (temp.Pos("&&", pos)) {
		char next = 0;
		
		try {
			next = temp[pos + 2];
		} catch (ExceptionRange< String::size_type>) {
			/* Out of range, throw different kind
			   of exception */
			throw ExceptionAssert("Misformed anchor at the end of the template");
		}
		
		if (next != '&') /* This anchor is not an escaped &  */
			throw ExceptionAssert("Unsubstituted anchor in template");
	}

	/* Replace all &&&s with & */
	temp.Subst("&&&", "&");
	
	return temp;
}

