/***********************************************************************

  ColorBox2: Widgets.h
    (c) 2004 by Martin Decky
  
  Distributed under the terms of Lesser General Public Licence, see
  COPYING.

  0.3			20th August 2004	initial

***********************************************************************/


#ifndef __CB2__WIDGETS_H__
#define __CB2__WIDGETS_H__


#include <cb2/utils.h>
#include <cb2/Template.h>


namespace CB2 {
	
	class Container;   /**< Forward declaration */
	class Application; /**< Forward declaration */
	
	
	/** Size values class
	 *
	 * Instances of this class hold scalar
	 * size (length) values including the units.
	 *
	 */
	class Size : public Class {
		public:
			
			/** Unit type
			 *
			 */
			typedef enum {
				px,
				em
			} size_type;
			
			Size(const int value, const size_type type); /**< Constructor */
			
			String GetSize() const; /**< Get size value including unit */
			
		private:
			
			int svalue;      /**< Scalar value */
			size_type stype; /**< Units */
	};
	
	
	/** Base class for all visual application parts
	 *
	 * A widget is any structural object of the application
	 * which has its visual representation on the web page
	 * of the web application.
	 *
	 */
	class Widget : public Class {
		public:
			Widget(Container & parent);   /**< Constructor */
			Widget(Application & parent); /**< Constructor */
			virtual ~Widget();            /**< Destructor */
			
		protected:
			void Data(const String & data); /**< Render widget */
			
		private:
			Widget(const Widget & prev);         /**< Copy constructor */
			void operator=(const Widget & prev); /**< Assignment operator */
			
			Template * ptempl; /**< Parent template */
			String panchor;    /**< Anchor in the parent template */
			bool filled;       /**< Widget rendered */
	};
	
	
	/** Base class for all widgets containing other widgets
	 *
	 * Containers store another widgets (even containers)
	 * is some sort of visual hierarchy.
	 *
	 */
	class Container : public Widget {
		public:
			Container(Container & parent);   /**< Constructor */
			Container(Application & parent); /**< Constructor */
			
			virtual Template * Add(Widget * child, String & anchor) = 0; /**< Add nested widget */
	};
	
	
	/** Horizontal spacer
	 *
	 * Empty widget creating horizontal space.
	 *
	 */
	class Spacer : public Widget {
		public:
			Spacer(Container & parent, const String & css_class = "", const String & css_id = "");   /**< Constructor */
			Spacer(Application & parent, const String & css_class = "", const String & css_id = ""); /**< Constructor */
			
			Spacer(Container & parent, const Size & height);   /**< Constructor */
			Spacer(Application & parent, const Size & height); /**< Constructor */
			
			virtual ~Spacer(); /**< Destructor */
		
		private:
		
			void init(const String & css_class, const String & css_id); /**< Initialize parameters */
			void init(const Size & height);                             /**< Initialize parameters */
			
			String data; /**< Spacer data */
			
	};
	
	
}


#endif
